@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -36,7 +36,7 @@ from nuitka.nodes.BuiltinIteratorNodes import (
     StatementSpecialUnpackCheck,
 )
 from nuitka.nodes.BuiltinNextNodes import ExpressionSpecialUnpack
-from nuitka.nodes.BuiltinRefNodes import makeExpressionBuiltinRef
+from nuitka.nodes.BuiltinRefNodes import makeExpressionBuiltinTypeRef
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
 from nuitka.nodes.CoroutineNodes import (
     ExpressionCoroutineObjectBody,
@@ -95,8 +95,7 @@ def _insertFinalReturnStatement(function_statements_body, return_statement):
 
 def _insertInitialSetLocalsDictStatement(function_body, function_statements_body):
     locals_statement = StatementSetLocalsDictionary(
-        locals_scope=function_body.getFunctionLocalsScope(),
-        source_ref=function_body.source_ref,
+        locals_scope=function_body.getLocalsScope(), source_ref=function_body.source_ref
     )
 
     if function_statements_body is None:
@@ -123,7 +122,7 @@ def _injectDecorator(decorators, inject, acceptable, source_ref):
             break
     else:
         decorators.append(
-            makeExpressionBuiltinRef(builtin_name=inject, source_ref=source_ref)
+            makeExpressionBuiltinTypeRef(builtin_name=inject, source_ref=source_ref)
         )
 
 
@@ -153,6 +152,7 @@ def buildFunctionNode(provider, node, source_ref):
                 name=node.name,
                 code_object=code_object,
                 flags=flags,
+                auto_release=None,
                 source_ref=source_ref,
             )
 
@@ -163,6 +163,7 @@ def buildFunctionNode(provider, node, source_ref):
                 name=node.name,
                 code_object=code_object,
                 flags=flags,
+                auto_release=None,
                 source_ref=source_ref,
             )
 
@@ -170,7 +171,7 @@ def buildFunctionNode(provider, node, source_ref):
 
         code_body.qualname_provider = provider
 
-        for variable in function_body.getVariables():
+        for variable in function_body.getProvidedVariables():
             code_body.getVariableForReference(variable.getName())
 
         function_body.setBody(
@@ -311,6 +312,7 @@ def buildAsyncFunctionNode(provider, node, source_ref):
             name=node.name,
             code_object=code_object,
             flags=flags,
+            auto_release=None,
             source_ref=source_ref,
         )
     else:
@@ -319,12 +321,13 @@ def buildAsyncFunctionNode(provider, node, source_ref):
             name=node.name,
             code_object=code_object,
             flags=flags,
+            auto_release=None,
             source_ref=source_ref,
         )
 
     function_body.qualname_provider = provider
 
-    for variable in creator_function_body.getVariables():
+    for variable in creator_function_body.getProvidedVariables():
         function_body.getVariableForReference(variable.getName())
 
     decorators = buildNodeList(
@@ -464,7 +467,7 @@ def buildParameterAnnotations(provider, node, source_ref):
         keys.append(mangle(key))
         values.append(value)
 
-    def extractArg(arg):
+    def extractArgAnnotation(arg):
         if getKind(arg) == "Name":
             assert arg.annotation is None
         elif getKind(arg) == "arg":
@@ -475,15 +478,19 @@ def buildParameterAnnotations(provider, node, source_ref):
                 )
         elif getKind(arg) == "Tuple":
             for sub_arg in arg.elts:
-                extractArg(sub_arg)
+                extractArgAnnotation(sub_arg)
         else:
             assert False, getKind(arg)
 
+    if python_version >= 380:
+        for arg in node.args.posonlyargs:
+            extractArgAnnotation(arg)
+
     for arg in node.args.args:
-        extractArg(arg)
+        extractArgAnnotation(arg)
 
     for arg in node.args.kwonlyargs:
-        extractArg(arg)
+        extractArgAnnotation(arg)
 
     if python_version < 340:
         if node.args.varargannotation is not None:
@@ -499,9 +506,9 @@ def buildParameterAnnotations(provider, node, source_ref):
             )
     else:
         if node.args.vararg is not None:
-            extractArg(node.args.vararg)
+            extractArgAnnotation(node.args.vararg)
         if node.args.kwarg is not None:
-            extractArg(node.args.kwarg)
+            extractArgAnnotation(node.args.kwarg)
 
     # Return value annotation (not there for lambdas)
     if hasattr(node, "returns") and node.returns is not None:
@@ -546,7 +553,7 @@ def _wrapFunctionWithSpecialNestedArgs(
         for element_index, arg_name in enumerate(arg_names):
             if getKind(arg_name) == "Name":
                 arg_var = outer_body.createProvidedVariable(arg_name.id)
-                outer_body.registerProvidedVariable(arg_var)
+                outer_body.getLocalsScope().registerProvidedVariable(arg_var)
 
                 statements.append(
                     StatementAssignmentVariable(
@@ -682,14 +689,15 @@ def buildFunctionWithParsing(
 
         return normal_args
 
-    normal_args = extractNormalArgs(node.args.args)
-
     parameters = ParameterSpec(
         ps_name=name,
-        ps_normal_args=normal_args,
+        ps_normal_args=extractNormalArgs(node.args.args),
+        ps_pos_only_args=[extractArg(arg) for arg in node.args.posonlyargs]
+        if python_version >= 380
+        else (),
         ps_kw_only_args=[extractArg(arg) for arg in node.args.kwonlyargs]
         if python_version >= 300
-        else [],
+        else (),
         ps_list_star_arg=extractArg(node.args.vararg),
         ps_dict_star_arg=extractArg(node.args.kwarg),
         ps_default_count=len(node.args.defaults),
@@ -706,6 +714,7 @@ def buildFunctionWithParsing(
         co_name=name,
         co_kind=function_kind,
         co_varnames=parameters.getParameterNames(),
+        co_freevars=(),
         co_argcount=parameters.getArgumentCount(),
         co_posonlyargcount=parameters.getPosOnlyParameterCount(),
         co_kwonlyargcount=parameters.getKwOnlyParameterCount(),
@@ -723,6 +732,7 @@ def buildFunctionWithParsing(
         flags=flags,
         doc=function_doc,
         parameters=parameters,
+        auto_release=None,
         source_ref=source_ref,
     )
 
@@ -751,7 +761,7 @@ def addFunctionVariableReleases(function):
     source_ref = function.getSourceReference()
 
     for variable in function.getLocalVariables():
-        # Shared variables are freed by function object attachment.
+        # Shared variables are freed by function attachment.
         if variable.getOwner() is not function:
             continue
 
@@ -770,5 +780,3 @@ def addFunctionVariableReleases(function):
         )
 
         function.setBody(makeStatementsSequenceFromStatement(statement=body))
-
-        # assert body.isStatementAborting(), body.asXmlText()
